/*
 * 文 件 名:  CommonUtils
 * 版    权:  浩鲸云计算科技股份有限公司
 * 描    述:  <描述>
 * 修 改 人:  lv.wenbin@huashe.com
 * 修改时间:  2019/2/27
 */
package com.huashe.citybrain.commons.base.utils;

import com.huashe.citybrain.commons.base.bean.Response;
import com.huashe.citybrain.commons.base.exception.Assert;
import com.huashe.citybrain.commons.base.exception.BusinessException;
import com.huashe.citybrain.commons.base.exception.ErrorCode;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.UncategorizedSQLException;

/**
 * 通用工具类想 <功能详细描述>
 *
 * @author lv.wenbin@huashe.com
 * @version [版本号, 2019/2/27]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public abstract class CommonUtils {
    private static final Logger log = LoggerFactory.getLogger(CommonUtils.class);

    /**
     * 线程睡眠
     *
     * @param millis 毫秒
     */
    public static void threadSleep(long millis) {
        try {
            Thread.sleep(millis);
        }
        catch (InterruptedException e) {
            log.error("Thread sleep fail!", e);
        }
    }

    /**
     * 响应载体内容提取
     *
     * @param response 响应
     * @param <T>      载体泛型
     * @return T
     */
    public static <T> T getData(Response<T> response) {
        Assert.notNull(response, ErrorCode.UNKNOWN_ERROR.getCode(), "响应为空!");
        boolean flag = (response.getCode() == ErrorCode.SUCCESS.getCode());
        Assert.isTrue(flag, response.getCode(), response.getMessage());
        return response.getData();
    }

    /**
     * 响应载体内容提取
     *
     * @param response 响应
     * @param <T>      载体泛型
     * @return T
     */
    public static <T> T getDataNoException(Response<T> response, T defaultObj) {
        T ret = null;

        try {
            Assert.notNull(response, ErrorCode.UNKNOWN_ERROR.getCode(), "响应为空!");
            boolean flag = (response.getCode() == ErrorCode.SUCCESS.getCode());
            Assert.isTrue(flag, response.getCode(), response.getMessage());
            ret = response.getData();
        }
        catch (BusinessException e) {
            ret = defaultObj;
            log.warn("fail!", e);
        }

        return ret;
    }

    /**
     * 封装业务异常设值
     *
     * @param response 响应
     * @param e        异常堆栈
     */
    public static void businessExceptionRes(Response<?> response, BusinessException e) {
        response.setCode(e.getCode());
        response.setMessage(e.getMessage());
    }

    /**
     * 未知业务异常设值
     *
     * @param response 响应
     * @param e        异常堆栈
     */
    public static void unknownExceptionRes(Response<?> response, Exception e) {
        response.setCode(ErrorCode.UNKNOWN_ERROR.getCode());
        response.setMessage(ErrorCode.UNKNOWN_ERROR.getMessage());
    }

    /**
     * Yml中文unicode解码
     *
     * @param tagName tagName
     * @return
     */
    public static String tagToString(String tagName) {
        String tmp = "-" + tagName;
        tmp = StringUtils.replace(tmp, "-", "\\");
        return UnicodeUtils.unicodeToString(tmp);
    }

    public static BusinessException convertException(UncategorizedSQLException e) {
        BusinessException bException = null;
        Throwable t = e.getCause();
        String msg = t.getMessage();

        if (StringUtils.contains(msg, "Memory left in process limi")) {
            bException = new BusinessException(ErrorCode.DB_ERROR.getCode(), "Impala 内存超限！", e);
        }
        else {
            bException = new BusinessException(ErrorCode.DB_ERROR.getCode(), "Sql 执行失败！", e);
        }

        return bException;
    }
}
