/*
 * 文 件 名:  ExcelUtils
 * 版    权:  浩鲸云计算科技股份有限公司
 * 描    述:  <描述>
 * 修 改 人:  learshaw
 * 修改时间:  2020/3/18
 * 跟踪单号:  <跟踪单号>
 * 修改单号:  <修改单号>
 * 修改内容:  <修改内容>
 */
package com.huashe.citybrain.commons.base.utils;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.apache.poi.ss.usermodel.CellBase;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Poi Excel工具
 * <功能详细描述>
 *
 * @author learshaw
 * @version [版本号, 2020/3/18]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public abstract class ExcelUtils {
    private static final Logger log = LoggerFactory.getLogger(ExcelUtils.class);

    /**
     * 写Excel <功能详细描述>
     *
     * @param workbook excel文档对象
     * @param path     存放路径
     * @see [类、类#方法、类#成员]
     */
    public static void writeXls(Workbook workbook, String path) {
        FileOutputStream outputStream = null;

        try {
            // 修改模板内容导出新模板
            outputStream = FileUtils.openOutputStream(new File(path));

            if (null != outputStream) {
                workbook.write(outputStream);
            }
        }
        catch (FileNotFoundException e) {
            log.error("write excel fail!", e);
        }
        catch (Exception e) {
            log.error("write excel fail!", e);
        }
        finally {
            CloseUtils.close(outputStream);
        }
    }

    /**
     * 获取输入流
     * <功能详细描述>
     *
     * @param workBook workBook
     * @return InputStream
     * @see [类、类#方法、类#成员]
     */
    public static InputStream getInputStream(Workbook workBook) {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        InputStream inputStream = null;

        try {
            workBook.write(byteArrayOutputStream);
            byte[] content = byteArrayOutputStream.toByteArray();
            inputStream = new ByteArrayInputStream(content);
        }
        catch (IOException e) {
            log.error("Write excel fail!" + ExceptionUtils.getStackTrace(e));
        }
        finally {
            CloseUtils.close(inputStream);
            CloseUtils.close(byteArrayOutputStream);
        }

        return inputStream;
    }

    /**
     * 获取字节数组输出流
     * <功能详细描述>
     *
     * @param workBook HSSFWorkbook
     * @return ByteArrayOutputStream
     * @see [类、类#方法、类#成员]
     */
    public static ByteArrayOutputStream getByteArrayOutputStream(Workbook workBook) {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();

        try {
            workBook.write(byteArrayOutputStream);
        }
        catch (IOException e) {
            log.error("Write excel fail!" + ExceptionUtils.getStackTrace(e));
        }
        finally {
            CloseUtils.close(byteArrayOutputStream);
        }

        return byteArrayOutputStream;
    }

    /**
     * 读取单元格值
     * <功能详细描述>
     *
     * @param cell 单元格
     * @return 值
     * @see [类、类#方法、类#成员]
     */
    public static String getCellStringValue(CellBase cell) {
        String cellValue = "";

        // 字符串类型
        if (cell.getCellType() == CellType.STRING) {
            cellValue = cell.getStringCellValue();
        }
        // 数值类型
        else if (cell.getCellType() == CellType.NUMERIC) {
            cellValue = String.valueOf(cell.getNumericCellValue());
        }
        // 公式
        else if (cell.getCellType() == CellType.FORMULA) {
            cellValue = String.valueOf(cell.getNumericCellValue());
        }
        else if (cell.getCellType() == CellType.BOOLEAN) {
            cellValue = String.valueOf(cell.getBooleanCellValue());
        }

        return cellValue;
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param cell     单元格
     * @param value    填充值
     * @param cellType 单元格类型
     * @see [类、类#方法、类#成员]
     */
    public static void setCellValue(CellBase cell, String value, CellType cellType) {
        cell.setCellType(cellType);

        if (CellType.STRING == cellType) {
            cell.setCellValue(value);
        }
        else if (CellType.NUMERIC == cellType) {
            cell.setCellValue(Integer.parseInt(value));
        }
        else if (CellType.FORMULA == cellType) {
            cell.setCellFormula(value);
        }
        else {
            cell.setCellValue(value);
        }
    }
}