/*
 * 文 件 名:  ExcelUtil.java
 * 版    权:  江苏智通交通科技
 * 描    述:  <描述>
 * 修 改 人:  wblv
 * 修改时间:  2014年8月12日
 * 跟踪单号:  <跟踪单号>
 * 修改单号:  <修改单号>
 * 修改内容:  <修改内容>
 */
package com.huashe.citybrain.commons.base.utils;

import com.huashe.citybrain.commons.base.bean.poi.CellStyle;
import com.huashe.citybrain.commons.base.bean.poi.SheetFooter;
import com.huashe.citybrain.commons.base.bean.poi.SheetMain;
import com.huashe.citybrain.commons.base.bean.poi.WorkbookConfig;
import com.huashe.citybrain.commons.base.exception.Assert;
import com.huashe.citybrain.commons.base.exception.BusinessException;
import com.huashe.citybrain.commons.base.exception.ErrorCode;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFFormulaEvaluator;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.CellBase;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.CellValue;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.RegionUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Excel操作工具
 *
 * @author wblv
 * @version [版本号, 2014年8月12日]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public abstract class ExcelXlsUtils {
    private static final Logger log = LoggerFactory.getLogger(ExcelXlsUtils.class);

    /**
     * 读取excel <功能详细描述>
     *
     * @param filePath 文件路径
     * @return HSSFWorkbook
     * @see [类、类#方法、类#成员]
     */
    public static HSSFWorkbook readXls(String filePath) {
        InputStream inputStream = null;
        HSSFWorkbook workBook = null;

        try {
            inputStream = new FileInputStream(filePath);
            workBook = new HSSFWorkbook(inputStream);
        }
        catch (FileNotFoundException e) {
            log.error("readXls fail!", e);
        }
        catch (Exception e) {
            log.error("readXls fail!", e);
        }
        finally {
            CloseUtils.close(inputStream);
        }

        return workBook;
    }

    /**
     * 写Excel <功能详细描述>
     *
     * @param hssfWorkBook excel文档对象
     * @param path         存放路径
     * @see [类、类#方法、类#成员]
     */
    public static void writeXls(HSSFWorkbook hssfWorkBook, String path) {
        ExcelUtils.writeXls(hssfWorkBook, path);
    }

    /**
     * 获取输入流
     * <功能详细描述>
     *
     * @param hssfWorkbook workBook
     * @return InputStream
     * @see [类、类#方法、类#成员]
     */
    public static InputStream getInputStream(HSSFWorkbook hssfWorkbook) {
        return ExcelUtils.getInputStream(hssfWorkbook);
    }

    /**
     * 获取字节数组输出流
     * <功能详细描述>
     *
     * @param hssfWorkbook HSSFWorkbook
     * @return ByteArrayOutputStream
     * @see [类、类#方法、类#成员]
     */
    public static ByteArrayOutputStream getByteArrayOutputStream(HSSFWorkbook hssfWorkbook) {
        return ExcelUtils.getByteArrayOutputStream(hssfWorkbook);
    }

    /**
     * 获取sheet <功能详细描述>
     *
     * @param workBook  Excel工作薄
     * @param sheetName Sheet名称
     * @return HSSFSheet
     * @see [类、类#方法、类#成员]
     */
    public static HSSFSheet getSheet(HSSFWorkbook workBook, String sheetName) {
        HSSFSheet sheet = workBook.getSheet(sheetName);

        if (null == sheet) {
            sheet = workBook.createSheet(sheetName);
        }

        return sheet;
    }

    /**
     * 获取行
     * <功能详细描述>
     *
     * @param rowIndex 行号
     * @return HSSFRow
     * @see [类、类#方法、类#成员]
     */
    public static HSSFRow getRow(HSSFSheet sheet, int rowIndex) {
        HSSFRow row = sheet.getRow(rowIndex);

        if (row == null) {
            row = sheet.createRow(rowIndex);
        }

        return row;
    }

    /**
     * 获取单元格
     * <功能详细描述>
     *
     * @param row      行对象
     * @param colIndex 列编号
     * @return HSSFCell
     * @see [类、类#方法、类#成员]
     */
    public static HSSFCell getCell(HSSFRow row, int colIndex) {
        HSSFCell cell = row.getCell(colIndex);

        if (cell == null) {
            cell = row.createCell(colIndex);
        }

        return cell;
    }

    /**
     * 获取单元格
     * <功能详细描述>
     *
     * @param sheet    sheet页对象
     * @param rowIndex 行编号
     * @param colIndex 列编号
     * @return HSSFCell
     * @see [类、类#方法、类#成员]
     */
    public static HSSFCell getCell(HSSFSheet sheet, int rowIndex, int colIndex) {
        HSSFRow row = getRow(sheet, rowIndex);
        return getCell(row, colIndex);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param sheet    Sheet名称
     * @param rowIndex 行坐标
     * @param colIndex 列坐标
     * @param value    填充值
     * @param cellType 单元格参数类型
     * @see [类、类#方法、类#成员]
     */
    public static void setSheetCellValue(HSSFSheet sheet, int rowIndex, int colIndex, String value, CellType cellType) {
        HSSFCell cell = getCell(sheet, rowIndex, colIndex);
        setCellValue(cell, value, cellType);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param sheet    Sheet名称
     * @param rowIndex 行坐标
     * @param colIndex 列坐标
     * @param value    填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setSheetCellValue(HSSFSheet sheet, int rowIndex, int colIndex, int value) {
        HSSFCell cell = getCell(sheet, rowIndex, colIndex);
        setCellValue(cell, value);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param sheet    Sheet名称
     * @param rowIndex 行坐标
     * @param colIndex 列坐标
     * @param value    填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setSheetCellValue(HSSFSheet sheet, int rowIndex, int colIndex, long value) {
        HSSFCell cell = getCell(sheet, rowIndex, colIndex);
        setCellValue(cell, value);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param sheet    Sheet名称
     * @param rowIndex 行坐标
     * @param colIndex 列坐标
     * @param value    填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setSheetCellValue(HSSFSheet sheet, int rowIndex, int colIndex, String value) {
        HSSFCell cell = getCell(sheet, rowIndex, colIndex);
        setCellValue(cell, value);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param cell     单元格
     * @param value    填充值
     * @param cellType 单元格类型
     * @see [类、类#方法、类#成员]
     */
    public static void setCellValue(CellBase cell, String value, CellType cellType) {
        ExcelUtils.setCellValue(cell, value, cellType);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param cell  单元格
     * @param value 填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setCellValue(HSSFCell cell, int value) {
        cell.setCellType(CellType.NUMERIC);
        cell.setCellValue(value);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param cell  单元格
     * @param value 填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setCellValue(HSSFCell cell, long value) {
        cell.setCellType(CellType.NUMERIC);
        cell.setCellValue(value);
    }

    /**
     * 填充单元格值 <功能详细描述>
     *
     * @param cell  单元格
     * @param value 填充值
     * @see [类、类#方法、类#成员]
     */
    public static void setCellValue(HSSFCell cell, String value) {
        cell.setCellType(CellType.STRING);
        cell.setCellValue(value);
    }

    public static String getSheetCellValue(HSSFSheet sheet, int rowIndex, int colIndex) {
        HSSFCell cell = getCell(sheet, rowIndex, colIndex);
        return getCellStringValue(cell);
    }

    /**
     * 读取单元格值
     * <功能详细描述>
     *
     * @param hssfCell 单元格
     * @return 值
     * @see [类、类#方法、类#成员]
     */
    public static String getCellStringValue(HSSFCell hssfCell) {
        return ExcelUtils.getCellStringValue(hssfCell);
    }

    /**
     * 合并单元格
     * <功能详细描述>
     *
     * @param sheet      sheet页
     * @param rowFrom    开始行
     * @param rowTo      结束行
     * @param columnFrom 开始列
     * @param columnTo   结束列
     * @see [类、类#方法、类#成员]
     */
    public static void mergeCell(HSSFSheet sheet, int rowFrom, int rowTo, int columnFrom, int columnTo) {
        CellRangeAddress crAddress = new CellRangeAddress(rowFrom, rowTo, columnFrom, columnTo);
        sheet.addMergedRegion(crAddress);
    }

    /**
     * 设置单元格格式
     * <功能详细描述>
     *
     * @see [类、类#方法、类#成员]
     */
    public static void setCellStyle(HSSFWorkbook workBook, HSSFCell cell, HorizontalAlignment styleType) {
        HSSFCellStyle cellStyle = workBook.createCellStyle();
        cellStyle.setAlignment(styleType);
        cell.setCellStyle(cellStyle);
    }

    /**
     * 计算出单元格中公式的值
     * <功能详细描述>
     *
     * @param workBook 工作部
     * @param cell     单元格
     * @see [类、类#方法、类#成员]
     */
    public static double calculateFormulaValue(HSSFWorkbook workBook, HSSFCell cell) {
        double retValue = 0.0;

        if (cell.getCellType() == CellType.FORMULA) {
            HSSFFormulaEvaluator evaluator = new HSSFFormulaEvaluator(workBook);
            CellValue cellValue = evaluator.evaluate(cell);
            retValue = cellValue.getNumberValue();
        }

        return retValue;
    }

    /**
     * 重设公式单元格值
     * <功能详细描述>
     *
     * @param sheet    sheet
     * @param rowIndex rowIndex
     * @Param colIndex colIndex
     * @see [类、类#方法、类#成员]
     */
    public static void resetFormulaCell(HSSFSheet sheet, int rowIndex, int colIndex) {
        HSSFCell hssfCell = getCell(sheet, rowIndex, colIndex);

        if (CellType.FORMULA == hssfCell.getCellType()) {
            //取得公式单元格的公式,重新设置
            hssfCell.setCellFormula(hssfCell.getCellFormula());
        }
    }

    /**
     * 重设公式单元格值
     * <功能详细描述>
     *
     * @param hssfCell hssfCell
     * @see [类、类#方法、类#成员]
     */
    public static void resetFormulaCell(HSSFCell hssfCell) {
        if (CellType.FORMULA == hssfCell.getCellType()) {
            //取得公式单元格的公式,重新设置
            hssfCell.setCellFormula(hssfCell.getCellFormula());
        }
    }

    /**
     * 字母列名转换为矩阵列坐标
     * <pre>
     * A -> [0]
     * B -> [1]
     * C -> [2]
     *  ......
     * IV -> [255]
     * </pre>
     *
     * @param letterName 字母坐标
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static int convertToColumnIndex(String letterName) {
        int column = -1;
        letterName = StringUtils.upperCase(letterName);

        for (int i = 0; i < letterName.length(); ++i) {
            int c = letterName.charAt(i);
            column = (column + 1) * 26 + c - 'A';
        }

        return column;
    }

    /**
     * 列座标转换为字母列名
     * <pre>
     * [0] -> A
     * [1] -> B
     * [2] -> C
     *  ......
     * [255] -> IV
     * </pre>
     *
     * @param columnIndex 矩阵坐标
     * @return
     * @see [类、类#方法、类#成员]
     */
    public static String convertToLetterName(int columnIndex) {
        columnIndex += 1;
        String strResult = "";
        int intRound = columnIndex / 26;
        int intMod = columnIndex % 26;

        if (intRound != 0) {
            strResult = String.valueOf(((char) (intRound + 64)));
        }

        strResult += String.valueOf(((char) (intMod + 64)));

        return strResult;
    }

    /**
     * 矩阵坐标转换为单元格编号
     * <pre>
     * [0][0] -> A1
     * [0][1] -> A2
     * ......
     * [255][65535] -> IV65536
     * </pre>
     *
     * @param columnIndex 列坐标
     * @param rowIndex    行坐标
     * @return String
     * @see [类、类#方法、类#成员]
     */
    public static String convertToCellId(int columnIndex, int rowIndex) {
        return convertToLetterName(columnIndex) + (rowIndex + 1);
    }

    /**
     * 删除sheet
     * <功能详细描述>
     *
     * @param workBook  workBook
     * @param sheetName sheetName
     * @see [类、类#方法、类#成员]
     */
    public static void delSheet(HSSFWorkbook workBook, String sheetName) {
        int sheetIndex = workBook.getSheetIndex(sheetName);
        workBook.removeSheetAt(sheetIndex);
    }

    /**
     * 默认字体
     */
    private static final String DEFAULT_FONT = "宋体";

    /**
     * 创建Excel文件
     *
     * @param file      文件
     * @param sheetMain sheet
     * @throws Exception 异常
     */
    public static void createExcelFile(File file, SheetMain sheetMain) throws IllegalAccessException, IOException {
        OutputStream outputStream = new FileOutputStream(file);
        checkSheet(sheetMain);

        // 1.创建工作簿
        HSSFWorkbook hssfWorkbook = new HSSFWorkbook();
        WorkbookConfig workbookConfig = buildWorkbookCfg(hssfWorkbook, sheetMain);

        // 2.创建工作表
        HSSFSheet sheet = hssfWorkbook.createSheet(sheetMain.getSheetName());
        buildSheet(sheetMain, sheet, workbookConfig);

        // 3.创建行
        // 3.1创建头标题行;并且设置头标题
        int currentRow = writeHeadRow(sheetMain, sheet, workbookConfig);
        // 3.2创建列标题;并且设置列标题
        writeHeadRow2(sheetMain, sheet, workbookConfig, currentRow);

        // 4.创建列表内容
        List<?> ts = sheetMain.getTs();
        currentRow = writeBodyRow(ts, sheetMain, sheet, workbookConfig, currentRow);

        // 5创建脚标内容
        writeSheetFoot(sheetMain, sheet, workbookConfig, currentRow);

        // 写入文件
        hssfWorkbook.write(outputStream);
    }

    private static void checkSheet(SheetMain sheetMain) {
        try {
            if (null != sheetMain) {
                Assert.notNull(sheetMain.getSheetTitle(), ErrorCode.PARAM_ERROR.getCode(), "创建EXCEL文件-主标题为空");
                Assert.notNull(sheetMain.getSheetHeads(), ErrorCode.PARAM_ERROR.getCode(), "创建EXCEL文件-列标题为空");
                Assert
                    .isTrue(sheetMain.getSheetHeads().size() != 0, ErrorCode.PARAM_ERROR.getCode(), "创建EXCEL文件-列标题为空");
                Assert.notNull(sheetMain.getTs(), ErrorCode.PARAM_ERROR.getCode(), "创建EXCEL文件-列内容为空");
                Assert.isTrue(sheetMain.getTs().size() != 0, ErrorCode.PARAM_ERROR.getCode(), "创建EXCEL文件-列内容为空");
            }
            else {
                throw new BusinessException(-1, "创建EXCEL文件-主对象为空");
            }
        }
        catch (BusinessException e) {
            log.warn("checkSheet", e);
        }
    }

    private static WorkbookConfig buildWorkbookCfg(HSSFWorkbook hssfWorkbook, SheetMain sheetMain) {
        WorkbookConfig config = new WorkbookConfig();
        int headWidth = sheetMain.getSheetHeads().size();
        config.setHeadWidth(headWidth);

        // 1.1创建合并单元格对象
        config.setCallRangeAddress(buildCallRangeAddress(sheetMain, headWidth));
        // 1.2头标题样式
        config.setHeadStyle(createHSSFCellStyle(hssfWorkbook, DEFAULT_FONT, sheetMain.getTitleCellStyle().getFontSize(),
            sheetMain.getTitleCellStyle().isFontBold(), sheetMain.getTitleCellStyle().isBorderFlag(),
            sheetMain.getTitleCellStyle().isAlignmentCenter(),
            sheetMain.getTitleCellStyle().isVerticalAlignmentCenter()));
        // 1.3列标题样式
        config.setColHeadStyle(
            createHSSFCellStyle(hssfWorkbook, DEFAULT_FONT, sheetMain.getHeadCellStyle().getFontSize(),
                sheetMain.getHeadCellStyle().isFontBold(), sheetMain.getHeadCellStyle().isBorderFlag(),
                sheetMain.getHeadCellStyle().isAlignmentCenter(),
                sheetMain.getHeadCellStyle().isVerticalAlignmentCenter()));
        // 1.4列样式
        config.setColStyle(createHSSFCellStyle(hssfWorkbook, DEFAULT_FONT, sheetMain.getTitleCellStyle().getFontSize(),
            sheetMain.getTitleCellStyle().isFontBold(), sheetMain.getTitleCellStyle().isBorderFlag(),
            sheetMain.getTitleCellStyle().isAlignmentCenter(),
            sheetMain.getTitleCellStyle().isVerticalAlignmentCenter()));
        // 1.5脚标样式
        config.setColFooterStyle(
            createHSSFCellStyle(hssfWorkbook, DEFAULT_FONT, sheetMain.getFooterCellStyle().getFontSize(),
                sheetMain.getFooterCellStyle().isFontBold(), sheetMain.getFooterCellStyle().isBorderFlag(),
                sheetMain.getFooterCellStyle().isAlignmentCenter(),
                sheetMain.getFooterCellStyle().isVerticalAlignmentCenter()));

        return config;
    }

    /**
     * 初始化行样式
     *
     * @param workbook                工作薄对象
     * @param fontName                字体名称：宋体
     * @param fontSize                字号大小：11
     * @param fontBold                是否加粗：true/false
     * @param borderFlag              是否边框：true/false
     * @param alignmentCenter         水平居中：true/false
     * @param verticalAlignmentCenter 垂直居中：true/false
     * @return HSSFCellStyle
     */
    private static HSSFCellStyle createHSSFCellStyle(HSSFWorkbook workbook, String fontName, short fontSize,
        boolean fontBold, boolean borderFlag, boolean alignmentCenter, boolean verticalAlignmentCenter) {
        HSSFCellStyle hssfCellStyle = workbook.createCellStyle();

        // 设置位置
        setAbsolutePosition(hssfCellStyle, alignmentCenter, verticalAlignmentCenter);
        // 设置边框
        setBorder(hssfCellStyle, borderFlag);
        // 设置字体
        setFont(workbook, hssfCellStyle, fontName, fontSize, fontBold);

        return hssfCellStyle;
    }

    private static void buildSheet(SheetMain sheetMain, HSSFSheet sheet, WorkbookConfig workbookConfig) {
        // 2.1加载合并单元格对象
        if (workbookConfig.getHeadWidth() != 0 && null != sheetMain.getSheetTitle()) {
            sheet.addMergedRegion(workbookConfig.getCallRangeAddress());
        }

        // 设置默认列宽
        if (null != sheetMain.getCellWidths() && sheetMain.getCellWidths().size() != 0) {
            for (int i = 0; i < sheetMain.getCellWidths().size(); i++) {
                sheet.setColumnWidth(i, ((int) sheetMain.getCellWidths().get(i)) * 256);
            }
        }
        else {
            sheet.setDefaultColumnWidth(15);
        }

        if (sheetMain.getTitleCellStyle().isBorderFlag()) {
            // 下边框
            RegionUtil.setBorderBottom(BorderStyle.THIN, workbookConfig.getCallRangeAddress(), sheet);
            // 左边框
            RegionUtil.setBorderLeft(BorderStyle.THIN, workbookConfig.getCallRangeAddress(), sheet);
            // 有边框
            RegionUtil.setBorderRight(BorderStyle.THIN, workbookConfig.getCallRangeAddress(), sheet);
            // 上边框
            RegionUtil.setBorderTop(BorderStyle.THIN, workbookConfig.getCallRangeAddress(), sheet);
        }
    }

    private static int writeHeadRow(SheetMain sheetMain, HSSFSheet sheet, WorkbookConfig workbookConfig) {
        int currentRow = 0;

        HSSFRow row = sheet.createRow(sheetMain.getStartRow());
        HSSFCell cell = row.createCell(sheetMain.getStartCell());
        // 加载单元格样式
        cell.setCellStyle(workbookConfig.getHeadStyle());

        if (null != sheetMain.getSheetTitle()) {
            cell.setCellValue(sheetMain.getSheetTitle());
            currentRow += 1;
        }

        return currentRow;
    }

    private static void writeHeadRow2(SheetMain sheetMain, HSSFSheet sheet, WorkbookConfig workbookConfig,
        int currentRow) {
        if (null != sheetMain.getSheetHeads() && sheetMain.getSheetHeads().size() != 0) {
            HSSFRow row2 = sheet.createRow(currentRow);

            List<String> titles = sheetMain.getSheetHeads();
            int len = titles.size();

            for (int i = 0; i < len; i++) {
                HSSFCell cell2 = row2.createCell(sheetMain.getStartCell() + i);
                // 加载单元格样式
                cell2.setCellStyle(workbookConfig.getColHeadStyle());
                cell2.setCellValue(titles.get(i));
            }
        }
    }

    private static int writeBodyRow(List<?> ts, SheetMain sheetMain, HSSFSheet sheet, WorkbookConfig workbookConfig,
        int currentRow) throws IllegalAccessException {
        int tmpRow = currentRow;

        if (CollectionUtils.isNotEmpty(ts)) {
            if (ts.get(0) instanceof Map) {
                tmpRow = writeBodyFromMap(ts, sheetMain, sheet, workbookConfig, currentRow);
            }
            else {
                tmpRow = writeBodyFromList(ts, sheetMain, sheet, workbookConfig, currentRow);
            }
        }

        return tmpRow;
    }

    private static int writeBodyFromMap(List<?> ts, SheetMain sheetMain, HSSFSheet sheet, WorkbookConfig workbookConfig,
        int currentRow) {
        int tmpRow = currentRow;

        for (Object obj : ts) {
            tmpRow += 1;
            int i = 0;
            HSSFRow row3 = sheet.createRow(sheetMain.getStartRow() + tmpRow);
            Map<String, Object> map = (Map<String, Object>) obj;
            Set<Map.Entry<String, Object>> set = map.entrySet();
            Iterator<Map.Entry<String, Object>> iterator = set.iterator();

            while (iterator.hasNext()) {
                Map.Entry entry = iterator.next();
                HSSFCell cell1 = row3.createCell(sheetMain.getStartCell() + i);
                cell1.setCellStyle(workbookConfig.getColStyle());
                cell1.setCellValue(String.valueOf(entry.getValue()));
                i++;
            }
        }

        return tmpRow;
    }

    private static int writeBodyFromList(List<?> ts, SheetMain sheetMain, HSSFSheet sheet,
        WorkbookConfig workbookConfig, int currentRow) throws IllegalAccessException {
        int tmpRow = currentRow;

        for (int j = 0; j < ts.size(); j++) {
            tmpRow += 1;
            HSSFRow row3 = sheet.createRow(sheetMain.getStartRow() + tmpRow);
            // 得到类对象
            Class shellContentCla = (Class) ts.get(j).getClass();
            /* 得到类中的所有属性集合 */
            Field[] shellContentField = shellContentCla.getDeclaredFields();
            List<Field> fields = new ArrayList<>();

            for (int i = 0; i < shellContentField.length; i++) {
                Field field = shellContentField[i];
                if (!StringUtils.equals(field.getName(), "serialVersionUID")) {
                    fields.add(field);
                }
            }

            for (int i = 0; i < fields.size(); i++) {
                Field field = fields.get(i);
                field.setAccessible(true);
                Object val = field.get(ts.get(j));

                // 创建数据行,前面有两行,头标题行和列标题行
                HSSFCell cell1 = row3.createCell(sheetMain.getStartCell() + i);
                cell1.setCellStyle(workbookConfig.getColStyle());
                cell1.setCellValue("");

                if (null != val) {
                    cell1.setCellValue(val.toString());
                }
            }
        }

        return tmpRow;
    }

    private static void writeSheetFoot(SheetMain sheetMain, HSSFSheet sheet, WorkbookConfig workbookConfig,
        int currentRow) {
        int tmpRow = currentRow + 1;
        List<SheetFooter> sheetFooters = sheetMain.getSheetFooters();

        if (null != sheetFooters) {
            HSSFRow row4 = sheet.createRow(tmpRow);
            for (int i = 0; i < sheetFooters.size(); i++) {
                SheetFooter sheetFooter = sheetFooters.get(i);
                HSSFCell cell1 = row4.createCell(sheetMain.getStartCell() + 2 * i);
                cell1.setCellStyle(workbookConfig.getColHeadStyle());
                cell1.setCellValue(sheetFooter.getName());

                HSSFCell cell2 = row4.createCell(sheetMain.getStartCell() + 2 * i + 1);
                cell2.setCellStyle(workbookConfig.getColFooterStyle());
                cell2.setCellValue(sheetFooter.getValue());
            }
        }
    }

    private static CellRangeAddress buildCallRangeAddress(SheetMain sheetMain, int headWidth) {
        CellRangeAddress callRangeAddress = null;

        if (null != sheetMain.getSheetTitle()) {
            // 起始行,结束行,起始列,结束列
            int lastCol = (headWidth + sheetMain.getStartCell()) - (headWidth == 0 ? 0 : 1);
            callRangeAddress = new CellRangeAddress(sheetMain.getStartRow(), sheetMain.getStartRow(),
                sheetMain.getStartCell(), lastCol);
        }

        return callRangeAddress;
    }

    private static void setFont(HSSFWorkbook workbook, HSSFCellStyle hssfCellStyle, String fontName, short fontSize,
        boolean fontBold) {
        // --创建字体
        HSSFFont font = workbook.createFont();
        font.setFontHeightInPoints(fontSize);
        // --设置字体名称
        font.setFontName(StringUtils.isBlank(fontName) ? fontName : "宋体");
        // --设置是否加粗
        font.setBold(fontBold);
        // 加载字体
        hssfCellStyle.setFont(font);
    }

    /**
     * 绝对位置设定
     *
     * @param hssfCellStyle           hssfCellStyle
     * @param alignmentCenter         alignmentCenter
     * @param verticalAlignmentCenter verticalAlignmentCenter
     */
    private static void setAbsolutePosition(HSSFCellStyle hssfCellStyle, boolean alignmentCenter,
        boolean verticalAlignmentCenter) {
        // --水平居中
        if (alignmentCenter) {
            hssfCellStyle.setAlignment(HorizontalAlignment.CENTER);
        }

        // --垂直居中
        if (verticalAlignmentCenter) {
            hssfCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        }
    }

    /**
     * 设置边框
     *
     * @param hssfCellStyle hssfCellStyle
     * @param borderFlag    borderFlag
     */
    private static void setBorder(HSSFCellStyle hssfCellStyle, boolean borderFlag) {
        // --创建边框
        if (borderFlag) {
            hssfCellStyle.setBorderTop(BorderStyle.THIN);
            hssfCellStyle.setBorderRight(BorderStyle.THIN);
            hssfCellStyle.setBorderBottom(BorderStyle.THIN);
            hssfCellStyle.setBorderLeft(BorderStyle.THIN);
            hssfCellStyle.setTopBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
            hssfCellStyle.setLeftBorderColor(HSSFColor.HSSFColorPredefined.BLACK.getIndex());
        }
    }

    /**
     * 创建样式对象
     *
     * @param fontSize                fontSize
     * @param fontBold                fontBold
     * @param borderFlag              borderFlag
     * @param alignmentCenter         alignmentCenter
     * @param verticalAlignmentCenter verticalAlignmentCenter
     * @return CellStyle
     */
    public static CellStyle createCellStyle(Short fontSize, Boolean fontBold, Boolean borderFlag,
        Boolean alignmentCenter, Boolean verticalAlignmentCenter) {
        CellStyle cellStyle = new CellStyle();

        if (null != fontSize) {
            cellStyle.setFontSize(fontSize);
        }

        if (null != fontBold) {
            cellStyle.setFontBold(fontBold);
        }

        if (null != borderFlag) {
            cellStyle.setBorderFlag(borderFlag);
        }

        if (null != alignmentCenter) {
            cellStyle.setAlignmentCenter(alignmentCenter);
        }

        if (null != verticalAlignmentCenter) {
            cellStyle.setVerticalAlignmentCenter(verticalAlignmentCenter);
        }

        return cellStyle;
    }

    public static <T> File export(String fileName, String sheetName, String sheetTitle, List<String> headers,
        List<T> data) throws IllegalAccessException, IOException {
        if (data.isEmpty()) {
            log.warn("需要导出的数据为空");
            return null;
        }
        else {
            File file = new File(System.getProperty("java.io.tmpdir") + "/" + fileName + ".xls");
            SheetMain<T> sheetMain = new SheetMain<>();
            sheetMain
                .init(0, 0, Arrays.asList(15, 25, 15, 15, 10, 25), createCellStyle((short) 16, true, true, true, true),
                    createCellStyle((short) 11, true, true, false, true),
                    createCellStyle((short) 11, false, true, false, true),
                    createCellStyle((short) 11, false, true, false, true));
            sheetMain.setSheetName(sheetName);
            sheetMain.setSheetTitle(sheetTitle);
            sheetMain.setSheetHeads(headers);
            sheetMain.setTs(data);
            createExcelFile(file, sheetMain);
            return file;
        }
    }
}